#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

# import interfaces
from pyams_security.interfaces import ISecurityManager, IDirectorySearchPlugin, SYSTEM_PREFIX
from pyams_security.zmi.interfaces import ISecurityManagerMenu
from pyams_skin.interfaces import IInnerPage, IPageHeader
from pyams_skin.interfaces.container import ITableElementEditor
from pyams_skin.layer import IPyAMSLayer
from pyams_utils.interfaces import MANAGE_SECURITY_PERMISSION
from pyams_zmi.interfaces.menu import IControlPanelMenu
from pyams_zmi.layer import IAdminLayer
from z3c.table.interfaces import IValues, IColumn
from zope.component.interfaces import ISite

# import packages
from pyams_form.form import AJAXEditForm
from pyams_form.group import NamedWidgetsGroup
from pyams_pagelet.pagelet import pagelet_config
from pyams_security.zmi.widget import OrderedPluginsFieldWidget
from pyams_skin.container import ContainerView
from pyams_skin.page import DefaultPageHeaderAdapter
from pyams_skin.table import BaseTable, DefaultElementEditorAdapter, ActionColumn, I18nColumn, TrashColumn
from pyams_skin.viewlet.menu import MenuItem
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.registry import query_utility
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url
from pyams_viewlet.manager import viewletmanager_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminDialogEditForm
from pyams_zmi.view import AdminView
from pyramid.url import resource_url
from pyramid.view import view_config
from z3c.form import field
from z3c.form.browser.checkbox import SingleCheckBoxFieldWidget
from z3c.table.column import GetAttrColumn
from zope.interface import implementer, Interface

from pyams_security import _


@adapter_config(context=(ISecurityManager, IAdminLayer, Interface), provides=ITableElementEditor)
class SecurityManagerTableElementEditor(DefaultElementEditorAdapter):
    """Security manager table element editor"""

    view_name = 'security-manager.html'
    modal_target = False

    @property
    def url(self):
        site = get_parent(self.context, ISite)
        return resource_url(site, self.request, 'admin#{0}'.format(self.view_name))


@viewlet_config(name='security-manager.menu', context=ISite, layer=IAdminLayer, manager=IControlPanelMenu,
                permission=MANAGE_SECURITY_PERMISSION, weight=5)
@viewletmanager_config(name='security-manager.menu', context=ISite, layer=IAdminLayer)
@implementer(ISecurityManagerMenu)
class SecurityManagerMenuItem(MenuItem):
    """Security manager menu"""

    label = _("Security")
    icon_class = 'fa-lock'
    url = '#security-manager.html'


class SecurityManagerPluginsTable(BaseTable):
    """Security manager plug-ins table"""

    id = 'security_manager_table'
    title = _("Authentication and users directory plug-ins")

    @property
    def data_attributes(self):
        manager = query_utility(ISecurityManager)
        attributes = super(SecurityManagerPluginsTable, self).data_attributes
        table_attrs = {'data-ams-location': absolute_url(manager, self.request),
                       'data-ams-delete-target': 'delete-plugin.json'}
        if 'table' in attributes:
            attributes['table'].update(table_attrs)
        else:
            attributes['table'] = table_attrs
        return attributes


@adapter_config(name='search', context=(Interface, IAdminLayer, SecurityManagerPluginsTable), provides=IColumn)
class SecurityManagerPluginsSearchColumn(ActionColumn):
    """Security manager plugins search column"""

    icon_class = 'fa fa-fw fa-search'
    icon_hint = _("See plug-in contents")

    url = "search.html"
    weight = 1

    def renderCell(self, item):
        if not IDirectorySearchPlugin.providedBy(item):
            return ''
        return super(SecurityManagerPluginsSearchColumn, self).renderCell(item)


@adapter_config(name='name', context=(Interface, IAdminLayer, SecurityManagerPluginsTable), provides=IColumn)
class SecurityManagerPluginsNameColumn(I18nColumn, GetAttrColumn):
    """Security manager plugins name column"""

    _header = _("Name")
    attrName = 'title'
    weight = 10


@adapter_config(name='trash', context=(Interface, IAdminLayer, SecurityManagerPluginsTable), provides=IColumn)
class SecurityManagerPluginsTrashColumn(TrashColumn):
    """Security manager plugins trash column"""

    icon_hint = _("Delete plug-in")
    permission = MANAGE_SECURITY_PERMISSION

    checker = lambda col, x: x.prefix != SYSTEM_PREFIX


@adapter_config(context=(ISite, IAdminLayer, SecurityManagerPluginsTable), provides=IValues)
class SecurityManagerValuesAdapter(ContextRequestViewAdapter):
    """Security manager values adapter"""

    @property
    def values(self):
        manager = query_utility(ISecurityManager)
        if manager is not None:
            return list(manager.values())
        return ()


@pagelet_config(name='security-manager.html', context=ISite, layer=IPyAMSLayer, permission=MANAGE_SECURITY_PERMISSION)
@implementer(IInnerPage)
class SecurityManagerView(AdminView, ContainerView):
    """Security manager view"""

    table_class = SecurityManagerPluginsTable


@adapter_config(context=(ISite, IAdminLayer, SecurityManagerView), provides=IPageHeader)
class SecurityManagerHeaderAdapter(DefaultPageHeaderAdapter):
    """Security manager header adapter"""

    icon_class = 'fa fa-fw fa-lock'
    subtitle = _("Security manager")


@viewlet_config(name='security-manager.properties.menu', context=ISite, layer=IAdminLayer,
                manager=ISecurityManagerMenu, permission=MANAGE_SECURITY_PERMISSION, weight=1)
class SecurityManagerPropertiesMenuItem(MenuItem):
    """Security manager properties menu"""

    label = _("Properties...")
    url = 'properties.html'
    modal_target = True

    def get_url(self):
        manager = query_utility(ISecurityManager)
        return resource_url(manager, self.request, self.url)


@pagelet_config(name='properties.html', context=ISecurityManager, layer=IPyAMSLayer,
                permission=MANAGE_SECURITY_PERMISSION)
class SecurityManagerEditForm(AdminDialogEditForm):
    """Security manager edit form"""

    title = _("System security manager")
    legend = _("Security manager properties")
    icon_css_class = 'fa fa-fw fa-lock'
    label_css_class = 'control-label col-md-5'
    input_css_class = 'col-md-7'

    fields = field.Fields(ISecurityManager)
    fields['enable_social_login'].widgetFactory = SingleCheckBoxFieldWidget
    fields['open_registration'].widgetFactory = SingleCheckBoxFieldWidget
    fields['credentials_plugins_names'].widgetFactory = OrderedPluginsFieldWidget
    fields['authentication_plugins_names'].widgetFactory = OrderedPluginsFieldWidget
    fields['directory_plugins_names'].widgetFactory = OrderedPluginsFieldWidget

    ajax_handler = 'properties.json'
    edit_permission = MANAGE_SECURITY_PERMISSION

    def getContent(self):
        return query_utility(ISecurityManager)

    def update(self):
        super(SecurityManagerEditForm, self).update()
        self.add_group(NamedWidgetsGroup(self, 'social_group', self.widgets,
                                         ('enable_social_login', 'social_users_folder',
                                          'authomatic_secret', 'social_login_use_popup'),
                                         legend=_("Enable social login?"),
                                         css_class='inner',
                                         switch=True,
                                         checkbox_switch=True,
                                         checkbox_field=ISecurityManager['enable_social_login']))
        self.add_group(NamedWidgetsGroup(self, 'registry_group', self.widgets,
                                         ('open_registration', 'users_folder'),
                                         legend=_("Enable free registration?"),
                                         css_class='inner',
                                         switch=True,
                                         checkbox_switch=True,
                                         checkbox_field=ISecurityManager['open_registration']))
        plugins_group = NamedWidgetsGroup(self, 'plugins_group', self.widgets,
                                          ('credentials_plugins_names', 'authentication_plugins_names',
                                           'directory_plugins_names'),
                                          legend=_("Plug-ins"),
                                          css_class='inner')
        plugins_group.label_css_class = 'control-label col-md-4'
        plugins_group.input_css_class = 'col-md-8'
        self.add_group(plugins_group)


@view_config(name='properties.json', context=ISecurityManager, request_type=IPyAMSLayer,
             permission=MANAGE_SECURITY_PERMISSION, renderer='json', xhr=True)
class SecurityManagerAJAXEditForm(AJAXEditForm, SecurityManagerEditForm):
    """Security manager edit form, AJAX view"""


@view_config(name='delete-plugin.json', context=ISecurityManager, request_type=IPyAMSLayer,
             permission=MANAGE_SECURITY_PERMISSION, renderer='json', xhr=True)
def delete_security_manager_plugin(request):
    """Delete plug-in from security manager"""
    translate = request.localizer.translate
    name = request.params.get('object_name')
    if not name:
        return {'status': 'message',
                'messagebox': {'status': 'error',
                               'content': translate(_("No provided object_name argument!"))}}
    if name not in request.context:
        return {'status': 'message',
                'messagebox': {'status': 'error',
                               'content': translate(_("Given plug-in name doesn't exist!"))}}
    del request.context[name]
    return {'status': 'success'}
